"use strict";

var axios = require("axios");

module.exports = {
  getCacheKeyForURLAndParams: function getCacheKeyForURLAndParams(url, params) {
    if (params) {
      return escape(url + JSON.stringify(params));
    } else {
      return escape(url);
    }
  },
  fetchThenValidateAndCache: function fetchThenValidateAndCache(dataURL, params, validateResultFn) {
    var cacheKey = this.getCacheKeyForURLAndParams(dataURL, params);
    var fetchParams = { params: params };
    if (params && params.timeout !== undefined) {
      fetchParams.timeout = Number(params.timeout);
      if (fetchParams.timeout <= 0) {
        fetchParams.timeout = 10000; // set default to 10 seconds
      }
    }

    return axios.get(dataURL, fetchParams).then(function (response) {
      var result = response.data;
      if (result && validateResultFn(result)) {
        saveCacheForKeyAndValue(cacheKey, result);
        return result;
      } else if (result === null) {
        throw new Error("Network requested result is null.");
      } else if (result && !validateResultFn(result)) {
        throw new Error("Network requested result is not valid.");
      }
      throw new Error("Network requested FAILED.");
    }).catch(function (error) {
      console.warn("DataFetcher: " + error);
      var recoverResult = getCachedForKey(cacheKey);
      if (!recoverResult) {
        console.warn("DataFetcher: no cache found for recover");
        throw error;
      }
      // console.log("DataFetcher: recovered from cache");
      return recoverResult;
    });
  },
  getCachedDataForURLParamsMaxAgeMinute: function getCachedDataForURLParamsMaxAgeMinute(dataURL, params, maxAgeMinute) {
    var _this = this;

    return new Promise(function (resolve) {
      resolve(_this.getCacheForKeyAndMaxAgeMinute(_this.getCacheKeyForURLAndParams(dataURL, params), maxAgeMinute));
    });
  },
  getCacheForKeyAndMaxAgeMinute: function getCacheForKeyAndMaxAgeMinute(key) {
    var maxAgeMinute = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 5;

    return getCachedForKeyWithMaxAge(key, maxAgeMinute * 60 * 1000);
  }
};

// cache functions

function getCachedForKeyWithMaxAge(key, maxAge) {
  try {
    var entry = JSON.parse(window.localStorage[key]);
    if (maxAge === 0 || maxAge && entry.ts < new Date().getTime() - maxAge) {
      return null;
    } else {
      return entry.value;
    }
  } catch (e) {
    return null;
  }
}

function saveCacheForKeyAndValue(key, value) {
  try {
    if (value && key) {
      window.localStorage[key] = JSON.stringify({
        value: value,
        ts: new Date().getTime()
      });
      return true;
    }
  } catch (e) {
    clearOldCache();
    return false;
  }
  return false;
}

function getCachedForKey(key) {
  try {
    var data = window.localStorage[key];
    if (data) {
      var entry = JSON.parse(data);
      return entry.value;
    }
  } catch (e) {
    return null;
  }
  return null;
}

function clearOldCache() {
  checkLocalStorageSize("before");
  // clear cache that older than 3 days
  var maxAge = 3 * 24 * 60 * 60 * 1000; // 3 days
  var now = new Date().getTime();

  Object.keys(localStorage).forEach(function (el) {
    try {
      if (JSON.parse(localStorage[el])) {
        var entry = JSON.parse(localStorage[el]);
        if (entry.ts && entry.ts < now - maxAge) {
          delete localStorage.removeItem(el);
        }
      }
    } catch (err) {}
  });
  checkLocalStorageSize("after");
}

function checkLocalStorageSize(moment) {
  var totalSize = Object.keys(localStorage).reduce(function (prev, el) {
    return prev + (localStorage[el].length + el.length) * 2;
  }, 0);
  // console.log(
  //   `Size ${moment} cleaning cahce: ${(totalSize / 1024).toFixed(2)}KB`
  // );
}